
#include "../include/Spi.h"

Spi::Spi(char *para_dev , int para_mode , int para_max_speed , int para_nbits , int para_first)
{
    mode = para_mode ;
    max_speed = para_max_speed ;
    nbits = para_nbits ;
    first = para_first ;

    //打开SPI设备
    spi_fd = open(para_dev , O_RDWR) ;
    if(spi_fd < 0)
    {
        perror("Error openig SPI device") ;
        printf("\n") ;
        ret = -1 ;
    }

    //设置SPI模式
    if( ( ioctl(spi_fd , SPI_IOC_WR_MODE , &mode) == -1 ) )
    {
        perror("Error is setting SPI mode") ;
        printf("\n") ;
        ret = -1 ;
    }
    if( ( ioctl(spi_fd , SPI_IOC_RD_MODE , &mode) == -1 ) )
    {
        perror("Error is setting SPI mode") ;
        printf("\n") ;
        ret = -1 ;
    }

    //设置SPI的续写速率
    if( ( ioctl(spi_fd , SPI_IOC_WR_MAX_SPEED_HZ , &max_speed) == -1 ) )
    {
        perror("Error is setting SPI Max Speed") ;
        printf("\n") ;
        ret = -1 ;
    }
    if( ( ioctl(spi_fd , SPI_IOC_RD_MAX_SPEED_HZ , &max_speed) == -1 ) )
    {
        perror("Error is setting SPI Max Speed") ;
        printf("\n") ;
        ret = -1 ;
    }

    //设置SPI一次读写位数
    if( ( ioctl(spi_fd , SPI_IOC_WR_BITS_PER_WORD , &nbits) == -1 ) )
    {
        perror("Error is setting SPI Nbits") ;
        printf("\n") ;
        ret = -1 ;
    }
    if( ( ioctl(spi_fd , SPI_IOC_RD_BITS_PER_WORD , &nbits) == -1 ) )
    {
        perror("Error is setting SPI Nbits") ;
        printf("\n") ;
        ret = -1 ;
    }

    //设置SPI高位先出
    if( ( ioctl(spi_fd , SPI_IOC_WR_LSB_FIRST , &first) == -1 ) )
    {
        perror("Error is setting SPI First") ;
        printf("\n") ;
        ret = -1 ;
    }
    if( ( ioctl(spi_fd , SPI_IOC_RD_LSB_FIRST , &first) == -1 ) )
    {
        perror("Error is setting SPI First") ;
        printf("\n") ;
        ret = -1 ;
    }
    pthread_mutex_init(&spi_mutex , NULL) ;
}

Spi::~Spi()
{
    close(spi_fd) ;
}

void Spi::spiMutexLock()
{
    pthread_mutex_lock(&spi_mutex) ;
}

void Spi::spiMutexUnlock()
{
    pthread_mutex_unlock(&spi_mutex) ;
}

void Spi::printTxBuffer()
{
    printf("\n==================== Tx Buffer Information ====================\n") ;
    for(int i = 0 ; i < sizeof(tx_buffer) ; i++)
    {
        printf("0x%02x\t" , tx_buffer[i]) ;
    }
    printf("\n==================== Tx Buffer Print End ====================\n") ;
}

void Spi::printRxBuffer()
{
    printf("\n==================== Rx Buffer Information ====================\n") ;
    for(int i = 0 ; i < sizeof(rx_buffer) ; i++)
    {
        printf("0x%02x\t" , rx_buffer[i]) ;
    }
    printf("\n==================== Rx Buffer Print End ====================\n") ;
}

int Spi::getTxLength()
{
    return sizeof(tx_buffer) ;
}

int Spi::getRxLength()
{
    return sizeof(rx_buffer) ;
}

uint8_t* Spi::getTxBuffer()
{
    return tx_buffer ;
}

uint8_t* Spi::getRxBuffer()
{
    return rx_buffer ;
}

int Spi::singleWriteSpi(unsigned char *para_tx_buffer , int para_tx_len)
{
    for(int i = 0 ; i < para_tx_len ; i++)
    {
        ret = write(spi_fd , &para_tx_buffer[i] , 1) ;
        if( ret == -1 )
        {
            printf("Error is singleWriteSpi write %d information is failed\n" , i) ;
        }

    }
    return 1 ;
}

int Spi::singleReadSpi(unsigned char *para_rx_buffer , int para_rx_len)
{
    for(int i = 0 ; i < para_rx_len ; i++)
    {
        ret = read(spi_fd , &para_rx_buffer[i] , 1) ;
        if( ret == -1 )
        {
            printf("Error is singleReadSpi read %d information is failed\n" , i) ;
        }
    }
    return 1 ;
}

int Spi::writeSpi(unsigned char *para_tx_buffer , int para_tx_len)
{
    ret = write( spi_fd , para_tx_buffer , para_tx_len) ;
    if( ret == -1 )
    {
        printf("Error is writeSpi is failed\n") ;
        return ret ;
    }
    return ret ;
}

int Spi::readSpi( unsigned char *para_rx_buffer , int para_rx_len)
{
    ret = read( spi_fd , para_rx_buffer , para_rx_len) ;
    if(ret == -1)
    {
        printf("Error is readSpi is failed\n") ;
        return ret ;
    }
    return ret ;
}

int Spi::readWriteSpi(unsigned char *para_tx_buffer , unsigned char *para_rx_buffer , int para_tx_len , int para_rx_len )
{
    int maxlength = ( (para_tx_len > para_rx_len) ? para_tx_len : para_rx_len  ) ;
    memset(para_rx_buffer , 0 , para_rx_len) ;
    struct spi_ioc_transfer transfer = {
        .tx_buf = (unsigned long)para_tx_buffer,
        .rx_buf = (unsigned long)para_rx_buffer,
        .len = (unsigned int ) maxlength,
        .speed_hz = (unsigned int)max_speed,
        .delay_usecs = 0,
        .bits_per_word = (unsigned char)nbits,
        .cs_change = 0,
    };

    ret = ioctl( spi_fd , SPI_IOC_MESSAGE(1) , &transfer) ;
    if(ret < 0)
    {
        perror("Error is transferring SPI message") ;
        printf("\n") ;
        return -1 ;
    }

    return 1 ;
}


void Spi::printTxBuffer(uint8_t *para_tx_buffer , int para_tx_len)
{
    printf("\n==================== Tx Buffer Information ====================\n") ;
    for(int i = 0 ; i < para_tx_len ; i++)
    {
        printf("0x%02x\t" , para_tx_buffer[i]) ;
    }
    printf("\n==================== Tx Buffer Print End ====================\n") ;
}
void Spi::printRxBuffer(uint8_t *para_rx_buffer , int para_rx_len)
{
    printf("\n==================== Rx Buffer Information ====================\n") ;
    for(int i = 0 ; i < para_rx_len ; i++)
    {
        printf("0x%02x\t" , para_rx_buffer[i]) ;
    }
    printf("\n==================== Rx Buffer Print End ====================\n") ;
}
