#include "../include/dio.h"

static int fp_di[8];
static int fp_do[8];

static char fname[64];

#define DIO_H 		"1"
#define DIO_L 		"0"

char *di_getfile(int n) {
	int id = 987 + n;
	memset(fname, 0, 64);
	sprintf(fname, "/sys/class/gpio/gpio%d/value", id);
	printf("FILE DI = %s\n", fname);
	return fname;
}

char *do_getfile(int n) {
	int id = 975 + n;
	memset(fname, 0, 64);
	sprintf(fname, "/sys/class/gpio/gpio%d/value", id);
	printf("FILE DO = %s\n", fname);
	return fname;
}

void dio_init(void) {
	for(int i=0; i<8; i++) {
		fp_di[i] = open(di_getfile(i),O_RDONLY);
		if(fp_di[i] < 0) {
			printf("### ERR: file init di[%d] fail\n", i);
		}
		fp_do[i] = open(do_getfile(i),O_WRONLY);
		//fp_do[i] = open(do_getfile(i),O_RDWR);
	}
}

void dio_write(int n, int v) {
	if(fp_do[n] < 0) {
		printf("### ERR: file write[%d] fail\n", n);
		return;
	}
	if(v) {
		write(fp_do[n], DIO_H, 2);
	}else{
		write(fp_do[n], DIO_L, 2);
	}
	return;
}

int dio_read(int n) {
	char vstr[4];

	if(fp_di[n] < 0) {
		printf("### ERR: file fail\n");
		return -1;
	}

	int ret = read(fp_di[n], vstr, 4);
	if(ret < 0) {
		printf("### ERR: file read[%d] fail\n", n);
	}

	return atoi(vstr);
}

void dio_close(void) {
}


void dio_test() {
	dio_init();

#if 1
	while(1) {
		for(int i=0; i<8; i++) {
			dio_write(i, 1);
//			usleep(1000*200);
			sleep(3);
			dio_write(i, 0);
//			usleep(1000*200);
			sleep(3);
		}
	}
#else
	int v[8];
	for(int i=0; i<8; i++) {
		v[i] = -1;
	}
	while(1) {
		for(int i=0; i<8; i++) {
			int ret = dio_read(i);
//			printf("DO[%d]=%d\n", i, v[i]);
			if(ret != v[i]) {
				v[i] = ret;
				printf("---------------->DO[%d]=%d\n", i, v[i]);
			}
		}
		usleep(1000*10);
	}
#endif
}



void initGpio(int pin)
{
    FILE * fp =fopen("/sys/class/gpio/export","w");
    if (fp == NULL)
    {
 
        perror("export open filed");
    }
    else
    {
        fprintf(fp,"%d",pin);
    }
    fclose(fp);
}
 
// direction="in" or "out"
void setGpioDirection(int pin,const char *direction)
{
    char path[100] = {0};
    sprintf(path,"/sys/class/gpio/gpio%d/direction",pin);
    FILE * fp =fopen(path,"w");
    if (fp == NULL)
    {
 
        perror("direction open filed");
    }
    else
    {
        fprintf(fp,"%s",direction);
    }
    fclose(fp);
}
 
int getGpioValue(int pin)
{
    char path[128];
    char value_str[3];
    int fd;
 
    snprintf(path, sizeof(path), "/sys/class/gpio/gpio%d/value", pin);
    fd = open(path, O_RDONLY);
    if (fd < 0)
    {
        perror("Failed to open gpio value!");
        return -1;
    }
 
    if (read(fd, value_str, 3) < 0)
    {
        perror("Failed to read value!");
        return -1;
    }
    close(fd);
//	printf("path: %s\n", path);
//	printf("read: %s\n", value_str);
    return (atoi(value_str));
}
void setGpioValue(int pin,int value)
{
    char path[128];
    snprintf(path, sizeof(path), "/sys/class/gpio/gpio%d/value", pin);
    FILE * fp =fopen(path,"w");
    if (fp != NULL)
    {
        fprintf(fp,"%d",value);
    }
    else
    {
        perror("Failed to open gpio value!");
    }
    fclose(fp);
}

void GpioTest(void) {
	int i;

	int pin_do = 975;
	for(i=0; i<8; i++) {
		initGpio(pin_do+i);
		setGpioDirection(pin_do+i,"out");
	}
	while(1) {
		for(i=0; i<8; i++) {
			setGpioValue(pin_do+i, 1);
		}
		sleep(1);
		for(i=0; i<8; i++) {
			setGpioValue(pin_do+i, 0);
		}
		sleep(1);
	}
}

void GpioReadTest(void) {
	int i;
	int pin_di = 987;
	for(int i=0; i<8; i++) {
		initGpio(pin_di+i);
		setGpioDirection(pin_di+i,"in");
	}

	int v[8];
	for(i=0; i<8; i++) {
		v[i] = -1;
	}
	while(1) {
		for(i=0; i<8; i++) {
			int ret = getGpioValue(pin_di+i);
//			printf("DO[%d]=%d\n", i, ret);
			if(ret != v[i]) {
				v[i] = ret;
				printf("---------------->DO[%d]=%d\n", i, v[i]);
			}
		}
		usleep(1000*10);
	}
}
 